/**
 * This software was developed at the National Institute of Standards and
 * Technology by employees of the Federal Government in the course of
 * their official duties. Pursuant to title 17 Section 105 of the United
 * States Code this software is not subject to copyright protection and
 * is in the public domain.
 * 
 * NIST assumes no responsibility whatsoever for its use by other parties,
 * and makes no guarantees, expressed or implied, about its quality,
 * reliability, or any other characteristic.
 * 
 * We would appreciate acknowledgement if the software is used.
 * 
 * NIST ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION AND
 * DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 * 
 * 
 * This software might use libraries that are under GNU public license or
 * other licenses. Please refer to the licenses of all libraries required 
 * by thsi software.
 *
 */
/**
 * @file mutex.c
 * @date Created: 01/11/2010
 */

#include "mutex.h"
#include "log.h"
#include <time.h>

#define LOGLEVEL LEVEL_DEBUG

inline bool initMutex(Mutex* self)
{
  LOG(LOGLEVEL, "([0x%08X] Mutex):  >  [0x%08X] INIT", pthread_self(), self);
  return (pthread_mutex_init(self, NULL) == 0);
}

inline void releaseMutex(Mutex* self)
{
  LOG(LOGLEVEL, "([0x%08X] Mutex):  >  [0x%08X] RELEASE", pthread_self(), self);
  (void)pthread_mutex_destroy(self);
}

inline void lockMutex(Mutex* self)
{
  LOG(LOGLEVEL, "([0x%08X] Mutex): --> [0x%08X] REQ LOCK ", pthread_self(), self);
  (void)pthread_mutex_lock(self);
  LOG(LOGLEVEL, "([0x%08X] Mutex): <-- [0x%08X] LOCKED", pthread_self(), self);
}

inline void unlockMutex(Mutex* self)
{
  LOG(LOGLEVEL, "([0x%08X] Mutex): ==> [0x%08X] UNLOCK", pthread_self(), self);
  (void)pthread_mutex_unlock(self);
  LOG(LOGLEVEL, "([0x%08X] Mutex): <== [0x%08X] UNLOCKED", pthread_self(), self);
}

inline void waitMutex(Mutex* self)
{
  LOG(LOGLEVEL, "([0x%08X] Mutex): >>> [0x%08X] WAIT", pthread_self(), self);
  lockMutex(self);
  unlockMutex(self);
  LOG(LOGLEVEL, "([0x%08X] Mutex): <<< [0x%08X] WAIT", pthread_self(), self);
}

inline int initCond(Cond *cond)
{
  LOG(LOGLEVEL, "([0x%08X] Condition):  >  [0x%08X] INIT", pthread_self(), cond );
  return (pthread_cond_init(cond, NULL) == 0);
}


inline int signalCond(Cond *cond)
{   
  LOG(LOGLEVEL, "([0x%08X] Condition signal): --> to [0x%08X] ", pthread_self(), cond);
  return (pthread_cond_signal(cond));
}


inline int waitCond(Cond *cond, Mutex *self)
{ 
  struct timespec to;
  memset(&to, 0, sizeof(to));
  to.tv_sec = time(0) +5;
  to.tv_nsec = 0;
  LOG(LOGLEVEL, "([0x%08X] Condition wait): --> [0x%08X] at Mutex[0x%08x] ", pthread_self(), cond, self);
  return(pthread_cond_timedwait(cond, self, &to));
  //return(pthread_cond_wait(cond, self));
}

inline int sem_wait_wrapper(sem_t *sem_var)
{ 
  struct timespec to;
  memset(&to, 0, sizeof(to));
  to.tv_sec = time(0) +10;	// 30 sec
  to.tv_nsec = 0;

  return sem_timedwait(sem_var, &to);	
}
