/**
 * This software was developed at the National Institute of Standards and
 * Technology by employees of the Federal Government in the course of
 * their official duties. Pursuant to title 17 Section 105 of the United
 * States Code this software is not subject to copyright protection and
 * is in the public domain.
 * 
 * NIST assumes no responsibility whatsoever for its use by other parties,
 * and makes no guarantees, expressed or implied, about its quality,
 * reliability, or any other characteristic.
 * 
 * We would appreciate acknowledgement if the software is used.
 * 
 * NIST ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION AND
 * DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 * 
 * 
 * This software might use libraries that are under GNU public license or
 * other licenses. Please refer to the licenses of all libraries required 
 * by thsi software.
 *
 */
/**
 * @file socket.h
 * @date Created: 12/29/2009
 *
 * Socket utility functions.
 *
 * Uses log.h for error reporting
 */

#ifndef __SOCKET_H__
#define __SOCKET_H__

#include <sys/socket.h>
#include "util/prefix.h"
#include "util/types.h"

/** 
 * Max. length of the textual representation of a socket.
 *
 * @see sockAddrToStr
 * @see socketToStr
 */
#define MAX_SOCKET_STRING_LEN (MAX_IP_V6_STR_LEN + 6)

/**
 * Reads \c num Bytes from a socket.
 * In case of an error, \c fd is closed and set to \c -1.
 *
 * @note Blocking call
 *
 * @param fd File-descriptor pointer
 * @param buffer (out) Destination for the read data
 * @param num Number of Bytes to read
 * @return \c true = successful, \c = failed
 * @see sendNum
 */
extern bool recvNum(int* fd, void* buffer, size_t num);

/** 
 * Writes \c num Bytes to a socket.
 * In case of an error, \c fd is closed and set to \c -1.
 *
 * @param fd File-descriptor pointer
 * @param buffer Data to write
 * @param num Size of buffer
 * @return \c true = successful, \c = failed
 */
extern bool sendNum(int* fd, void* buffer, size_t num);

/**
 * Returns a textual representation of a \c sockaddr.
 *
 * @param sa Socket address
 * @param dest Destination buffer
 * @param size Size of \c dest
 * @return String 
 */
extern const char* sockAddrToStr(const struct sockaddr* sa, 
                                 char* dest, size_t size);

/**
 * Returns a textual representation of a socket.
 *
 * @param fd Socket (file descriptor)
 * @param isPeer Is a conncted peer (= client)
 * @param dest Destination buffer
 * @param size Size of \c dest
 * @return String 
 */
extern const char* socketToStr(int fd, bool isPeer, char* dest, size_t size);

#endif // !__SOCKET_H__

